<?php

namespace SuperRhino\AppStoreServerApi\Response;

use JsonSerializable;
use ReflectionClass;
use ReflectionProperty;

abstract class AbstractResponse implements JsonSerializable {

    protected array $exceptKeys = [];

    protected array $onlyKeys = [];

    public function __construct(array $parameters = []) {
        foreach ($parameters as $name => $value) {
            $this->{$name} = $this->parameterConverter($name, $value);
        }
    }

    /**
     * @param array $parameters
     * @return static
     */
    public static function make(array $parameters = []) {
        return new static($parameters);
    }

    abstract protected function parameterConverter($name, $value);

    protected function arrayableMap($value, $key) {
        return $value;
    }

    public function jsonSerialize() {
        return $this->toArray();
    }

    public function __toString(): string {
        $json = json_encode($this->jsonSerialize());

        /*if (json_last_error() !== JSON_ERROR_NONE) {
            throw JsonEncodingException::forModel($this, json_last_error_msg());
        }*/

        return $json;
    }

    public function toArray(): array {
        $data = [];

        $class = new ReflectionClass(static::class);

        $properties = $class->getProperties(ReflectionProperty::IS_PUBLIC);

        foreach ($properties as $reflectionProperty) {
            // Skip static properties
            if ($reflectionProperty->isStatic()) {
                continue;
            }

            if (!$reflectionProperty->isInitialized($this)) {
                continue;
            }

            if (count($this->onlyKeys)) {
                if (!in_array($reflectionProperty->getName(), $this->onlyKeys)) {
                    continue;
                }
            } else {
                if (in_array($reflectionProperty->getName(), $this->exceptKeys)) {
                    continue;
                }
            }

            $data[$reflectionProperty->getName()] = $this->arrayableMap($reflectionProperty->getValue($this), $reflectionProperty->getName());
        }

        return $data;
    }
}