<?php

namespace SuperRhino\AppStoreServerApi\Response;

use SuperRhino\AppStoreServerApi\Enum\EnvironmentEnum;

class ServerNotificationsPayloadData extends AbstractResponse implements EnvironmentInterface {

    /**
     * The unique identifier of the app that the notification applies to.
     * This property is available for apps that are downloaded from the App Store; it isn’t present in the sandbox environment.
     * @var int
     */
    public int $appAppleId;

    /**
     * The bundle identifier of the app.
     * @var string
     */
    public string $bundleId;

    /**
     * The version of the build that identifies an iteration of the bundle.
     * @var string
     */
    public string $bundleVersion;

    /**
     * Transaction information signed by the App Store, in JSON Web Signature format.
     * @var JWSTransaction
     */
    public JWSTransaction $signedTransactionInfo;

    /**
     * Subscription renewal information signed by the App Store, in JSON Web Signature format.
     * @var JWSRenewalInfo
     */
    public JWSRenewalInfo $signedRenewalInfo;

    /**
     * The server environment that the notification applies to, either sandbox or production.
     * @var string
     * @see EnvironmentEnum
     */
    public string $environment;

    public function parameterConverter($name, $value) {
        switch ($name) {
            case 'signedTransactionInfo':
                return new JWSTransaction($value);
            case 'signedRenewalInfo':
                return new JWSRenewalInfo($value);
            default:
                return $value;
        }
    }

    public function isSandbox(): bool {
        return $this->environment == EnvironmentEnum::SANDBOX;
    }

    public function isProduction(): bool {
        return $this->environment == EnvironmentEnum::PRODUCTION;
    }

    protected function arrayableMap($value, $key) {
        if ($value instanceof  JWSTransaction || $value instanceof JWSRenewalInfo) {
            return $value->toArray();
        } else {
            return $value;
        }
    }


}